@@ -1,6 +1,16 @@
-#### Superclass Methods for all sparse logical matrices
+## METHODS FOR CLASS: lsparseMatrix (virtual)
+## sparse matrices with 'x' slot of type "logical" (TRUE, FALSE, or NA)
+## ... but _excluding_ ldiMatrix (FIXME?)
+## ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
+## MJ: no longer needed ... now inherited from Matrix
+if(FALSE) {
+setMethod("image", "lsparseMatrix",
+          function(x, ...) image(as(x, "dMatrix"), ...))
+} ## MJ
 
+## MJ: no longer needed ... replacement in ./sparseMatrix.R
+if(FALSE) {
 C2l <- function(from) {
     if(extends(cld <- getClassDef(class(from)), "lsparseMatrix"))
 	return(from)
@@ -22,11 +32,8 @@ C2l <- function(from) {
 setAs("CsparseMatrix", "lMatrix", C2l)
 setAs("CsparseMatrix", "lsparseMatrix", C2l)
 
-setAs("lsparseMatrix", "matrix",
-      function(from) as(as(from, "ldenseMatrix"), "matrix"))
-
 setAs("lsparseMatrix", "dsparseMatrix", function(from) as(from, "dMatrix"))
 
-
-
-setMethod("image", "lsparseMatrix", function(x, ...) image(as(x,"dMatrix"), ...))
+setAs("lsparseMatrix", "matrix",
+      function(from) as(as(from, "ldenseMatrix"), "matrix"))
+} ## MJ
