@@ -551,7 +551,7 @@ csi *cs_counts (const cs *A, const csi *parent, const csi *post, csi ata)
         if (parent [j] != -1) colcount [parent [j]] += colcount [j] ;
     }
     return (cs_idone (colcount, AT, w, 1)) ;    /* success: free workspace */
-}
+} 
 /* p [0..n] = cumulative sum of c [0..n-1], and then copy p [0..n-1] into c */
 double cs_cumsum (csi *p, csi *c, csi n)
 {
@@ -761,7 +761,7 @@ static csi cs_nonzero (csi i, csi j, double aij, void *other)
 csi cs_dropzeros (cs *A)
 {
     return (cs_fkeep (A, &cs_nonzero, NULL)) ;  /* keep all nonzero entries */
-}
+} 
 /* remove duplicate entries from A */
 csi cs_dupl (cs *A)
 {
@@ -1072,6 +1072,7 @@ csn *cs_lu (const cs *A, const css *S, double tol)
             }
         }
         if (ipiv == -1 || a <= 0) return (cs_ndone (N, NULL, xi, x, 0)) ;
+        /* tol=1 for  partial pivoting; tol<1 gives preference to diagonal */
         if (pinv [col] < 0 && fabs (x [col]) >= a*tol) ipiv = col ;
         /* --- Divide by pivot ---------------------------------------------- */
         pivot = x [ipiv] ;          /* the chosen pivot */
@@ -1250,6 +1251,13 @@ csi *cs_maxtrans (const cs *A, csi seed)  /*[jmatch [0..m-1]; imatch [0..n-1]]*/
     for (i = 0 ; i < m ; i++) if (jmatch [i] >= 0) imatch [jmatch [i]] = i ;
     return (cs_idone (jimatch, (m2 < n2) ? C : NULL, w, 1)) ;
 }
+
+#if csi == int
+# define csi_MAX INT_MAX
+#else
+# error "Need INT_MAX analogue of  csi type"
+#endif
+
 /* C = A*B */
 cs *cs_multiply (const cs *A, const cs *B)
 {
@@ -1268,10 +1276,12 @@ cs *cs_multiply (const cs *A, const cs *B)
     Cp = C->p ;
     for (j = 0 ; j < n ; j++)
     {
-        if (nz + m > C->nzmax && !cs_sprealloc (C, 2*(C->nzmax)+m))
+        if (C->nzmax > (csi_MAX - m)/2 || // 2*C->nzmax + m  overflows
+            (nz + m > C->nzmax && !cs_sprealloc (C, 2*(C->nzmax)+m)))
         {
+            warning("Too many non-zeros in sparse product: Out of memory");
             return (cs_done (C, w, x, 0)) ;             /* out of memory */
-        }
+        } 
         Ci = C->i ; Cx = C->x ;         /* C->i and C->x may be reallocated */
         Cp [j] = nz ;                   /* column j of C starts here */
         for (p = Bp [j] ; p < Bp [j+1] ; p++)
@@ -1360,35 +1370,35 @@ csi cs_print (const cs *A, csi brief)
 {
     csi p, j, m, n, nzmax, nz, *Ap, *Ai ;
     double *Ax ;
-    if (!A) { Rprintf ("(null)\n") ; return (0) ; }
+    if (!A) { printf ("(null)\n") ; return (0) ; }
     m = A->m ; n = A->n ; Ap = A->p ; Ai = A->i ; Ax = A->x ;
     nzmax = A->nzmax ; nz = A->nz ;
-    Rprintf ("CSparse Version %d.%d.%d, %s.  %s\n", CS_VER, CS_SUBVER,
+    printf ("CSparse Version %d.%d.%d, %s.  %s\n", CS_VER, CS_SUBVER,
         CS_SUBSUB, CS_DATE, CS_COPYRIGHT) ;
     if (nz < 0)
     {
-        Rprintf ("%g-by-%g, nzmax: %g nnz: %g, 1-norm: %g\n", (double) m,
+        printf ("%g-by-%g, nzmax: %g nnz: %g, 1-norm: %g\n", (double) m,
             (double) n, (double) nzmax, (double) (Ap [n]), cs_norm (A)) ;
         for (j = 0 ; j < n ; j++)
         {
-            Rprintf ("    col %g : locations %g to %g\n", (double) j,
+            printf ("    col %g : locations %g to %g\n", (double) j, 
                 (double) (Ap [j]), (double) (Ap [j+1]-1)) ;
             for (p = Ap [j] ; p < Ap [j+1] ; p++)
             {
-                Rprintf ("      %g : %g\n", (double) (Ai [p]), Ax ? Ax [p] : 1) ;
-                if (brief && p > 20) { Rprintf ("  ...\n") ; return (1) ; }
+                printf ("      %g : %g\n", (double) (Ai [p]), Ax ? Ax [p] : 1) ;
+                if (brief && p > 20) { printf ("  ...\n") ; return (1) ; }
             }
         }
     }
     else
     {
-        Rprintf ("triplet: %g-by-%g, nzmax: %g nnz: %g\n", (double) m,
+        printf ("triplet: %g-by-%g, nzmax: %g nnz: %g\n", (double) m,
             (double) n, (double) nzmax, (double) nz) ;
         for (p = 0 ; p < nz ; p++)
         {
-            Rprintf ("    %g %g : %g\n", (double) (Ai [p]), (double) (Ap [p]),
+            printf ("    %g %g : %g\n", (double) (Ai [p]), (double) (Ap [p]),
                 Ax ? Ax [p] : 1) ;
-            if (brief && p > 20) { Rprintf ("  ...\n") ; return (1) ; }
+            if (brief && p > 20) { printf ("  ...\n") ; return (1) ; }
         }
     }
     return (1) ;
@@ -1405,12 +1415,12 @@ csi cs_pvec (const csi *p, const double *b, double *x, csi n)
 csn *cs_qr (const cs *A, const css *S)
 {
     double *Rx, *Vx, *Ax, *x,  *Beta ;
-    csi i, k, p, m, n, vnz, p1, top, m2, len, col, rnz, *s, *leftmost, *Ap, *Ai,
+    csi i, k, p, n, vnz, p1, top, m2, len, col, rnz, *s, *leftmost, *Ap, *Ai,
         *parent, *Rp, *Ri, *Vp, *Vi, *w, *pinv, *q ;
     cs *R, *V ;
     csn *N ; // the result
     if (!CS_CSC (A) || !S) return (NULL) ;
-    m = A->m ; n = A->n ; Ap = A->p ; Ai = A->i ; Ax = A->x ;
+    n = A->n ; Ap = A->p ; Ai = A->i ; Ax = A->x ;
     q = S->q ; parent = S->parent ; pinv = S->pinv ; m2 = S->m2 ;
     vnz = S->lnz ; rnz = S->unz ; leftmost = S->leftmost ;
     w = cs_malloc (m2+n, sizeof (csi)) ;            /* get csi workspace */
@@ -1528,6 +1538,11 @@ csi cs_qrsol (csi order, const cs *A, double *b)
 /* return a random permutation vector, the identity perm, or p = n-1:-1:0.
  * seed = -1 means p = n-1:-1:0.  seed = 0 means p = identity.  otherwise
  * p = random permutation.  */
+/*
+ * NB: We use R's RNG *and* its state; i.e., if seed is not -1 or 0,
+ * ==  'seed' is *not* used at all in this version of cs_randperm() !
+*/
+
 csi *cs_randperm (csi n, csi seed)
 {
     csi *p, k, j, t ;
@@ -1536,14 +1551,16 @@ csi *cs_randperm (csi n, csi seed)
     if (!p) return (NULL) ;             /* out of memory */
     for (k = 0 ; k < n ; k++) p [k] = n-k-1 ;
     if (seed == -1) return (p) ;        /* return reverse permutation */
-    srand (seed) ;                      /* get new random number seed */
+    GetRNGstate();/* <- for R package Matrix
+    srand (seed) ;                      .* get new random number seed */
     for (k = 0 ; k < n ; k++)
     {
-        j = k + (rand ( ) % (n-k)) ;    /* j = rand integer in range k to n-1 */
+        j = k + (int)(unif_rand() * (n-k)); // j = rand integer in range k to n-1
         t = p [j] ;                     /* swap p[k] and p[j] */
         p [j] = p [k] ;
         p [k] = t ;
     }
+    PutRNGstate(); // <- R package Matrix
     return (p) ;
 }
 /* xi [top...n-1] = nodes reachable from graph of G*P' via nodes in B(:,k).
@@ -1922,6 +1939,7 @@ csi cs_sprealloc (cs *A, csi nzmax)
     csi ok, oki, okj = 1, okx = 1 ;
     if (!A) return (0) ;
     if (nzmax <= 0) nzmax = (CS_CSC (A)) ? (A->p [A->n]) : A->nz ;
+    nzmax = CS_MAX (nzmax, 1) ;
     A->i = cs_realloc (A->i, nzmax, sizeof (csi), &oki) ;
     if (CS_TRIPLET (A)) A->p = cs_realloc (A->p, nzmax, sizeof (csi), &okj) ;
     if (A->x) A->x = cs_realloc (A->x, nzmax, sizeof (double), &okx) ;
